@@ -74,6 +74,13 @@ from . import (
     RT_MESSAGETABLE,
     STD_OUTPUT_HANDLE,
     WAIT_FAILED,
+    ENABLE_VIRTUAL_TERMINAL_PROCESSING,
+    GENERIC_READ,
+    GENERIC_WRITE,
+    FILE_SHARE_READ,
+    FILE_SHARE_WRITE,
+    FILE_ATTRIBUTE_NORMAL,
+    OPEN_EXISTING,
     multi_str_from_addr,
     cmdline_from_args,
     ns_from_struct,
@@ -156,10 +163,13 @@ PFHANDLE = POINTER(FHANDLE)
 
 class SECURITY_ATTRIBUTES(ctypes.Structure):
     _fields_ = (
-        ("lpSecurityDescriptor", PVOID),
         ("nLength", DWORD),
+        ("lpSecurityDescriptor", PVOID),
         ("bInheritHandle", BOOL),
-    )
+        )
+
+    def __init__(self):
+        self.nLength = ctypes.sizeof(self)
 
 PSECURITY_ATTRIBUTES = POINTER(SECURITY_ATTRIBUTES)
 
@@ -193,6 +203,25 @@ def CreateFile(file_name, access, share_mode, sec_attr, dispo, flags, template):
     hdl.raise_on_invalid()
     return hdl
 
+def create_file(
+        file_name,
+        access=GENERIC_READ | GENERIC_WRITE,
+        share_mode=FILE_SHARE_READ | FILE_SHARE_WRITE,
+        flags=FILE_ATTRIBUTE_NORMAL,
+        dispo=OPEN_EXISTING,
+        sec_attr=None,
+        template=None
+        ):
+    return CreateFile(
+        file_name,
+        access,
+        share_mode,
+        sec_attr,
+        dispo,
+        flags,
+        template
+        )
+
 ################################################################################
 
 class _DUMMY_OVRLPD_STRUCT(ctypes.Structure):
@@ -265,17 +294,22 @@ def DeviceIoControl(hdl, ioctl, in_bytes, out_len):
 
 ################################################################################
 
-_GetCurrentProcess = fun_fact(_k32.GetCurrentProcess, (HANDLE,))
-
-def GetCurrentProcess():
-    return _GetCurrentProcess()
+GetCurrentProcess = fun_fact(_k32.GetCurrentProcess, (HANDLE,))
 
 ################################################################################
 
-_GetCurrentProcessId = fun_fact(_k32.GetCurrentProcessId, (DWORD,))
+GetCurrentProcessId = fun_fact(_k32.GetCurrentProcessId, (DWORD,))
 
-def GetCurrentProcessId():
-    return _GetCurrentProcessId()
+################################################################################
+
+_ProcessIdToSessionId = fun_fact(
+    _k32.ProcessIdToSessionId, (BOOL, DWORD, PDWORD)
+    )
+
+def ProcessIdToSessionId(pid):
+    session = DWORD()
+    raise_on_zero(_ProcessIdToSessionId(pid, ref(session)))
+    return session.value
 
 ################################################################################
 
@@ -821,18 +855,38 @@ def create_process(
 
 ################################################################################
 
+def _get_dir(func, order):
+    buf_size = 256
+    while True:
+        buf = ctypes.create_unicode_buffer(buf_size)
+        req_size = func(*((buf, buf_size) if order else (buf_size, buf)))
+        if req_size <= buf_size:
+            return buf.value
+        buf_size = req_size
+
+################################################################################
+
 _GetSystemDirectory = fun_fact(_k32.GetSystemDirectoryW, (UINT, PWSTR, UINT))
 
 def GetSystemDirectory():
-    buf_size = 256
-    buf = ctypes.create_unicode_buffer(buf_size)
-    req_size = _GetSystemDirectory(buf, buf_size)
-    if req_size <= buf_size:
-        return buf.value
-    buf = ctypes.create_unicode_buffer(req_size)
-    req_size = _GetSystemDirectory(buf, buf_size)
-    raise_if(req_size > buf_size)
-    return buf.value
+    return _get_dir(_GetSystemDirectory, 1)
+
+################################################################################
+
+_GetCurrentDirectory = fun_fact(
+    _k32.GetCurrentDirectoryW,
+    (DWORD, DWORD, PWSTR)
+    )
+
+def GetCurrentDirectory():
+    return _get_dir(_GetCurrentDirectory, 0)
+
+################################################################################
+
+_SetCurrentDirectory = fun_fact(_k32.SetCurrentDirectoryW, (BOOL, PWSTR))
+
+def SetCurrentDirectory(path):
+    raise_on_zero(_SetCurrentDirectory(path))
 
 ################################################################################
 
@@ -960,7 +1014,6 @@ def _EnumResNameCb(hmod, typ, name, ctxt):
     # keep on enumerating if the callback fails to return a value
     return res if res is not None else True
 
-
 _EnumResourceNames = fun_fact(
     _k32.EnumResourceNamesW,
     (BOOL, HANDLE, PWSTR, _EnumResNameCallback, CallbackContextPtr)
@@ -1264,3 +1317,46 @@ def cls(hdl=None):
     clear_screen(hdl)
 
 ################################################################################
+
+_GetConsoleMode = fun_fact(
+    _k32.GetConsoleMode, (BOOL, HANDLE, PDWORD)
+    )
+
+def GetConsoleMode(hdl):
+    mode = DWORD()
+    raise_on_zero(_GetConsoleMode(hdl, ref(mode)))
+    return mode.value
+
+################################################################################
+
+_SetConsoleMode = fun_fact(
+    _k32.SetConsoleMode, (BOOL, HANDLE, DWORD)
+    )
+
+def SetConsoleMode(hdl, mode):
+    raise_on_zero(_SetConsoleMode(hdl, mode))
+
+################################################################################
+
+def enable_virt_term(hdl=None):
+    if hdl is None:
+        hdl = GetStdHandle(STD_OUTPUT_HANDLE)
+        if GetFileType(hdl) != FILE_TYPE_CHAR:
+            return
+    mode = GetConsoleMode(hdl) | ENABLE_VIRTUAL_TERMINAL_PROCESSING
+    SetConsoleMode(hdl, mode)
+
+################################################################################
+
+SetErrorMode = fun_fact(_k32.SetErrorMode, (UINT, UINT))
+
+################################################################################
+
+_SetThreadErrorMode = fun_fact(_k32.SetThreadErrorMode, (BOOL, DWORD, PDWORD))
+
+def SetThreadErrorMode(mode):
+    old = DWORD()
+    raise_on_zero(_SetThreadErrorMode(mode, ref(old)))
+    return old.value
+
+################################################################################
