@@ -41,10 +41,12 @@ from .wtypes import (
     LONG_PTR,
     LPARAM,
     LRESULT,
+    NTSTATUS,
     PDWORD,
     POINT,
     POINTER,
     PPOINT,
+    PPVOID,
     PRECT,
     PVOID,
     PWSTR,
@@ -81,8 +83,13 @@ from . import (
     SPI_GETWORKAREA,
     SPIF_UPDATEINIFILE,
     SPIF_SENDCHANGE,
+    WM_QUIT,
+    )
+from .ntdll import (
+    RtlNtStatusToDosError,
+    proc_path_from_pid,
+    STATUS_BUFFER_TOO_SMALL,
     )
-from .ntdll import proc_path_from_pid
 
 _usr = ctypes.WinDLL("user32.dll")
 
@@ -567,10 +574,11 @@ def LockWorkStation():
 
 ################################################################################
 
-_GetShellWindow = fun_fact(_usr.GetShellWindow, (HWND,))
+GetDesktopWindow = fun_fact(_usr.GetDesktopWindow, (HWND,))
 
-def GetShellWindow():
-    return _GetShellWindow()
+################################################################################
+
+GetShellWindow = fun_fact(_usr.GetShellWindow, (HWND,))
 
 ################################################################################
 
@@ -725,6 +733,9 @@ class MSG(ctypes.Structure):
         ("pt", POINT)
         )
 
+    def __bool__(self):
+        return self.message != WM_QUIT
+
 PMSG = POINTER(MSG)
 
 ################################################################################
@@ -1446,11 +1457,11 @@ class NMHDR(ctypes.Structure):
         )
 PNMHDR = POINTER(NMHDR)
 
-
 MSDN_FIRST = 0xf060       # ModelesS Dialog
 MSDN_LAST = MSDN_FIRST + 50
 
 MSDN_ACTIVATE = MSDN_FIRST + 1
+
 class NM_MSD_ACTIVATE(ctypes.Structure):
     _fields_ = (
         ("hdr", NMHDR),
@@ -1503,3 +1514,57 @@ def EndDialog(hdlg, result):
     raise_on_zero(_EndDialog(hdlg, result))
 
 ################################################################################
+
+try:
+    _w32 = ctypes.WinDLL("win32u.dll")
+    _NtUserBuildHwndList = fun_fact(
+        _w32.NtUserBuildHwndList, (
+            NTSTATUS,
+            HANDLE,
+            HWND,
+            BOOL,
+            BOOL,
+            DWORD,
+            DWORD,
+            PPVOID,
+            PDWORD
+            )
+        )
+
+    # build_wnd_list(0, 0) -> EnumWindows(...)
+    # build_wnd_list(parent, 0) -> EnumChildWindows(parent, ...)
+    # build_wnd_list(0, tid) -> EnumThreadWindows(tid, ...)
+
+    def build_wnd_list(parent_wnd, thread_id, hdesk=0, hide_immersive=True):
+        enum_children = bool(parent_wnd)
+        allocated = 512
+        while True:
+            array = (allocated * PVOID)()
+            received = DWORD()
+            status = _NtUserBuildHwndList(
+                hdesk,
+                parent_wnd,
+                enum_children,
+                hide_immersive,
+                thread_id,
+                allocated,
+                array,
+                ref(received)
+                )
+            received = received.value
+            if status == 0:
+                break
+
+            if status == STATUS_BUFFER_TOO_SMALL:
+                # avoid under-allocating due to newly added windows -> + 32
+                allocated = received + 32
+            else:
+                raise ctypes.WinError(RtlNtStatusToDosError(status))
+
+        return array[:received - 1]
+
+except (FileNotFoundError, AttributeError):
+    def build_wnd_list(parent_wnd, thread_id, hdesk=0, hide_immersive=True):
+        raise NotImplementedError
+
+################################################################################
